/*
 * This is Tampa Ping-Pong convers/conversd derived from the wampes
 * convers package written by Dieter Deyke <deyke@mdddhd.fc.hp.com>
 * Modifications by Fred Baumgarten <dc6iq@insu1.etec.uni-karlsruhe.de>
 *
 * Modifications by Brian A. Lantz/KO4KS <brian@lantz.com>
 */

#ifndef CONVERSD_H
#define CONVERSD_H

#include <time.h>
#include <netinet/in.h>

#ifndef SOMAXCONN
#define SOMAXCONN       5
#endif

#ifndef WNOHANG
#define WNOHANG         1
#endif

#ifndef min
#define min(a,b) ((a) >= (b) ? (b) : (a))
#define max(a,b) ((a) >= (b) ? (a) : (b))
#endif

#if defined (BAN_ZERO)
#define MINCHANNEL	1
#else
#define MINCHANNEL	0
#endif

#define MAXCHANNEL      32767
#define uchar(x) ((x) & 0xff)

struct mbuf {
	struct mbuf *next;
	char *data;
};

#define NULLCHAR ((char *) 0)

struct clist {
	int channel;		/* channel number */
	int channelop;		/* channel operator status */
	time_t time;		/* join time */
	struct clist *next;	/* pointer to next entry */
};

#define NULLCLIST ((struct clist *) 0)

#define NAMESIZE (64 - 1)
#define PERSSIZE (512 - 1)
#define AWAYSIZE (512 - 1)
#define NOTIFYSIZE (512 - 1)

struct connection {
	int type;			/* Connection type */
#define CT_UNKNOWN      0
#define CT_USER         1
#define CT_HOST         2
#define CT_CLOSED       3
#define CT_OBSERVER	4
	char name[NAMESIZE + 1];	/* Name of user or host */
	char host[NAMESIZE + 1];	/* Name of host where user is logged on */
	char rev[NAMESIZE + 1];		/* revision of software (CT_HOST) */
	char nickname[NAMESIZE + 1];	/* Nickname of user */
	struct connection *via;		/* Pointer to neighbor host */
	time_t time;			/* Connect time */
	int locked;			/* Set if mesg already sent */
	
	/* These for local connections only: */
	int fd;				/* Socket descriptor */
	struct sockaddr_in sin;		/* Socket address */
	char ibuf[2048];		/* Input buffer */
	int icnt;			/* Number of bytes in input buffer */
	struct mbuf *obuf;		/* Output queue */
	long received;			/* Number of bytes received */
	long xmitted;			/* Number of bytes transmitted */
	int hostallowed;		/* connectee is allowed to become a HOST */
	
	/* For users: */
	int channel;			/* current channel number */
	struct clist *chan_list;	/* linked list of joined channels */
	char away[AWAYSIZE + 1];	/* Away string */
	time_t atime;			/* time of last "away" state change */
	time_t mtime;			/* time of last message receive */
	char pers[PERSSIZE + 1];	/* Personal string */
	int verbose;			/* verbose mode */
	char prompt[4];			/* prompt mode */
	char query[NAMESIZE + 1];	/* name of queried user */
	char notify[NOTIFYSIZE + 1];	/* List of calls you like to be notified */
	int charset_in;			/* charset - default ISO (ansi) */
	int charset_out;		/* charset - default ISO (ansi) */
	int channelop;			/* channel operator of current channel ? */
	int operator;			/* convers operator */
	int expected;			/* expected answer */
	int invitation_channel;		/* last invitation was from this channel */
	int width;			/* user screen width */
	int observer;			/* is the user an observer only */
	
	/* For links: */
	int amprnet;			/* is it /..UDAT (1) or /..USER (0) ? */
	int oldaway;			/* is it the pre3.02 Version of away ? */
	int features;
#define FEATURE_AWAY	1		/* a - "away feature" */
#define FEATURE_FWD	2		/* d - "destination forwarding" */
#define FEATURE_MODES	4		/* m - "channel modes" */
#define FEATURE_LINK	8		/* p - "ping pong link measurement" */
#define FEATURE_UDAT	16		/* u - "udat command extension and user command understood both" */
#define FEATURE_NICK	32		/* n - "TNOS Nickname extensions" */
	
	struct connection *next;	/* Linked list pointer */
};

#define CM_UNKNOWN   (1 << CT_UNKNOWN)
#define CM_USER      (1 << CT_USER)
#define CM_HOST      (1 << CT_HOST)
#define CM_CLOSED    (1 << CT_CLOSED)
#define CM_OBSERVER  (1 << CT_OBSERVER)

#define NULLCONNECTION  ((struct connection *) 0)

#define DEF_NR_PERMLINKS 40

#define HOSTNAMESIZE (80 - 1)
#define PCMDSIZE (256 - 1)

struct permlink {
	char name[HOSTNAMESIZE + 1];	/* Name of host */
	char *host;
	int port;
	struct connection *connection;	/* Pointer to associated connection */
	struct permlink *primary;	/* If !NULL, points to link we backup for */
/*	struct permlink *backup;	If !NULL, points to our backup link */
	time_t statetime;		/* Time of last (dis)connect */
	int tries;		/* Number of connect tries */
	long waittime;		/* Time between connect tries */
	time_t retrytime;	/* Time of next connect try */
	long testwaittime;	/* Time between tries */
	time_t testnexttime;	/* Time of next test */
	long rxtime;		/* rtt by other side */
	long txtime;		/* rtt found out by me */
	int quality;		/* quality of given link */
	int group;		/* link list group */
	int permanent;		/* this is a permanent entered link */
};

#define NULLPERMLINK  ((struct permlink *) 0)
#define NULLPERMARRAY  ((struct permlink **) 0)


extern struct permlink **permarray;

struct destination {
	char name[HOSTNAMESIZE + 1];	/* destination name */
	char rev[NAMESIZE + 1];	/* revision of software (CT_HOST) */
	struct permlink *link;	/* link to this destination */
	long rtt;		/* round trip time to this host */
	long last_sent_rtt;	/* last donwnstream sent rtt */
	int connected;		/* is it connected now? (used by notify_destinations) */
	int downstream[DEF_NR_PERMLINKS];	/* all up/downstream times */
	struct destination *next;	/* a one dimansional list is ok for now :-) */
};

#define NULLDESTINATION  ((struct destination *) 0)

struct channel {
	char name[80];		/* ascii name of channel */
#define TOPICSIZE (512 - 1)
	char topic[TOPICSIZE + 1];	/* topic of channel */
	time_t time;			/* when was the topic set ? */
	char tsetby[NAMESIZE + 1];	/* who set the topic? */
	int chan;		/* integer value of channel (<32768 !) */
	int flags;		/* channel flags */
#define M_CHAN_S 0x01		/* secret channel - number invisible */
#define M_CHAN_P 0x02		/* private channel - only join by invitation */
#define M_CHAN_T 0x04		/* topic settable by operator only */
#define M_CHAN_I 0x08		/* invisible channel - only sysops can see */
#define M_CHAN_M 0x10		/* moderated channel - operators may talk */
#define M_CHAN_L 0x20		/* local channel - no forwarding */
	struct channel *next;	/* linked list again */
};

#define NULLCHANNEL  ((struct channel *) 0)

struct cmdtable {
	char *name;
	void (*fnc)(struct connection *cp);
	int states;
};

extern struct cmdtable cmdtable[];


struct extendedcmds {
	char *name;
	struct extendedcmds *next;
};

#define NULLEXTCMD ((struct extendedcmds *)0)
extern struct extendedcmds *ecmds;


struct stats {
	long rx;
	long tx;
	time_t start;
};

#define ONE_HOUR (60*60)
#define ONE_DAY (60*60*24)
#define ONE_K (1024L)
#define ONE_MEG (1024L * 1024L)

#define L_SENT 1
#define L_RECV 2

extern void linklog(struct connection *p, int direction, const char *fmt, ...);

extern void appendstring(struct connection *cp, const char *string);
extern void appendc(struct connection *cp, const int n, const int ast);
extern void appendprompt(struct connection *cp, const int ast);
extern void destroy_channel(int number);
extern void free_connection(struct connection *cp);
extern int queuelength(const struct mbuf *bp);
extern struct permlink *find_permlink(const char *name);
extern void free_closed_connections(void);
extern char *getarg(char *line, int all);
extern char *getargcs(char *line, int all);
extern void getTXname(struct connection *cp, char *buf);
extern char *formatline(char *prefix, char *text, int linelen);
extern void notify_destinations(struct destination *dest, const char *host, const char *rev, int state);
extern char *ts(time_t gmt);
extern char *ts2(time_t gmt);
extern char *ts3(time_t seconds, char *buffer);
extern char *ts4(time_t seconds);
extern struct connection *alloc_connection(int fd);
extern void accept_connect_request(int flisten);
extern void clear_locks(void);
extern int count_user(int channel);
extern int count_topics(void);
extern void send_awaymsg(char *name, char *host, time_t time, char *text);
extern void send_mode(struct channel *chan);
extern void send_opermsg(char *name, char *host, char *from, int channel);
extern void send_persmsg(char *name, char *host, int channel, char *text, time_t time);
extern void send_topic(char *name, char *host, time_t time, int channel, char *text);
extern void send_user_change_msg(char *name, char *host, int oldchannel, int newchannel, char *pers, time_t time, int observer);
extern void send_msg_to_user(char *fromname, char *toname, char *text);
extern void send_msg_to_channel(char *fromname, int channel, char *text);
extern void send_invite_msg(char *fromname, char *toname, int channel);
extern void log_user_change(char *name, char *host, int oldchannel, int newchannel);
extern void update_destinations(struct permlink *p, char *name, long rtt, char *rev);
extern struct permlink *update_permlinks(char *name, struct connection *cp, int isperm);
extern void connect_permlinks(void);
extern int get_hash(char *name);
extern int ecmd_exists(char *cmdname);
extern void update_ecmds(void);
extern void update_stats(time_t now);
extern void init_stats(void);
extern void mysighup(int sig);
extern int dohelper(char *name, const char *search);

extern void route_cleanup(struct connection *cp);
extern int dest_check(struct connection *cp, struct destination *dest);
extern int user_check(struct connection *cp, char *user);

extern struct connection *connections;
extern struct permlink *permlinks;
extern struct permlink *notpermlinks;
extern struct destination *destinations;
extern struct channel *channels;
extern struct stats daily[];
extern struct stats hourly[];
extern time_t starttime;
extern time_t nextdailystats;
extern time_t nexthourlystats;
extern long hoursonline;
extern long daysonline;

extern time_t boottime;
extern time_t currtime;
extern char userfile[];
extern char motdfile[];
extern char restrictfile[];
extern char accessfile[];
extern char noaccessfile[];
extern char helpfile[];
extern char observerfile[];
extern char convtype[];
extern char myrev[];
extern char myfeatures[];
extern int restrictedmode;
extern char OBSERVERID[];
extern int NR_PERMLINKS;

extern void setargp(char *str);


#endif

